@@ -54,6 +54,8 @@ from elasticapm.utils.module_import import import_string
 
 __all__ = ("Client",)
 
+CLIENT_SINGLETON = None
+
 
 class Client(object):
     """
@@ -108,6 +110,8 @@ class Client(object):
             for msg in config.errors.values():
                 self.error_logger.error(msg)
             config.disable_send = True
+        if config.service_name == "python_service":
+            self.logger.warning("No custom SERVICE_NAME was set -- using non-descript default 'python_service'")
         self.config = VersionedConfig(config, version=None)
 
         # Insert the log_record_factory into the logging library
@@ -191,21 +195,31 @@ class Client(object):
             self._metrics.register(path)
         if self.config.breakdown_metrics:
             self._metrics.register("elasticapm.metrics.sets.breakdown.BreakdownMetricSet")
+        if self.config.prometheus_metrics:
+            self._metrics.register("elasticapm.metrics.sets.prometheus.PrometheusMetrics")
         self._thread_managers["metrics"] = self._metrics
         compat.atexit_register(self.close)
         if self.config.central_config:
             self._thread_managers["config"] = self.config
         else:
             self._config_updater = None
+        if self.config.use_elastic_excepthook:
+            self.original_excepthook = sys.excepthook
+            sys.excepthook = self._excepthook
         if config.enabled:
             self.start_threads()
 
+        # Save this Client object as the global CLIENT_SINGLETON
+        set_client(self)
+
     def start_threads(self):
-        with self._thread_starter_lock:
-            current_pid = os.getpid()
-            if self._pid != current_pid:
+        current_pid = os.getpid()
+        if self._pid != current_pid:
+            with self._thread_starter_lock:
                 self.logger.debug("Detected PID change from %r to %r, starting threads", self._pid, current_pid)
-                for manager_type, manager in self._thread_managers.items():
+                for manager_type, manager in sorted(
+                    self._thread_managers.items(), key=lambda item: item[1].start_stop_order
+                ):
                     self.logger.debug("Starting %s thread", manager_type)
                     manager.start_thread(pid=current_pid)
                 self._pid = current_pid
@@ -291,8 +305,10 @@ class Client(object):
     def close(self):
         if self.config.enabled:
             with self._thread_starter_lock:
-                for _, manager in self._thread_managers.items():
+                for _, manager in sorted(self._thread_managers.items(), key=lambda item: item[1].start_stop_order):
                     manager.stop_thread()
+        global CLIENT_SINGLETON
+        CLIENT_SINGLETON = None
 
     def get_service_info(self):
         if self._service_info:
@@ -536,7 +552,7 @@ class Client(object):
                     exc_name = "%s.%s" % (exc_module, exc_type)
                 else:
                     exc_name = exc_type
-                self.logger.info("Ignored %s exception due to exception type filter", exc_name)
+                self.logger.debug("Ignored %s exception due to exception type filter", exc_name)
                 return True
         return False
 
@@ -559,6 +575,14 @@ class Client(object):
             locals_processor_func=locals_processor_func,
         )
 
+    def _excepthook(self, type_, value, traceback):
+        try:
+            self.original_excepthook(type_, value, traceback)
+        except Exception:
+            self.capture_exception(handled=False)
+        finally:
+            self.capture_exception(exc_info=(type_, value, traceback), handled=False)
+
     def load_processors(self):
         """
         Loads processors from self.config.processors, as well as constants.HARDCODED_PROCESSORS.
@@ -571,6 +595,13 @@ class Client(object):
         # setdefault has the nice property that it returns the value that it just set on the dict
         return [seen.setdefault(path, import_string(path)) for path in processors if path not in seen]
 
+    def should_ignore_url(self, url):
+        if self.config.transaction_ignore_urls:
+            for pattern in self.config.transaction_ignore_urls:
+                if pattern.match(url):
+                    return True
+        return False
+
     def check_python_version(self):
         v = tuple(map(int, platform.python_version_tuple()[:2]))
         if v == (2, 7):
@@ -590,3 +621,15 @@ class DummyClient(Client):
 
     def send(self, url, **kwargs):
         return None
+
+
+def get_client():
+    return CLIENT_SINGLETON
+
+
+def set_client(client):
+    global CLIENT_SINGLETON
+    if CLIENT_SINGLETON:
+        logger = get_logger("elasticapm")
+        logger.warning("Client object is being set more than once", stack_info=True)
+    CLIENT_SINGLETON = client
