@@ -34,7 +34,6 @@ import re
 import threading
 import time
 import timeit
-import warnings
 from collections import defaultdict
 
 from elasticapm.conf import constants
@@ -42,11 +41,10 @@ from elasticapm.conf.constants import LABEL_RE, SPAN, TRANSACTION
 from elasticapm.context import init_execution_context
 from elasticapm.metrics.base_metrics import Timer
 from elasticapm.utils import compat, encoding, get_name_from_func
-from elasticapm.utils.deprecation import deprecated
 from elasticapm.utils.disttracing import TraceParent, TracingOptions
 from elasticapm.utils.logging import get_logger
 
-__all__ = ("capture_span", "tag", "label", "set_transaction_name", "set_custom_context", "set_user_context")
+__all__ = ("capture_span", "label", "set_transaction_name", "set_custom_context", "set_user_context")
 
 error_logger = get_logger("elasticapm.errors")
 logger = get_logger("elasticapm.traces")
@@ -116,33 +114,22 @@ class BaseSpan(object):
         labels = encoding.enforce_label_format(labels)
         self.labels.update(labels)
 
-    @deprecated("transaction/span.label()")
-    def tag(self, **tags):
-        """
-        This method is deprecated, please use "label()" instead.
-
-        Tag this span with one or multiple key/value tags. Both the values should be strings
-
-            span_obj.tag(key1="value1", key2="value2")
-
-        Note that keys will be dedotted, replacing dot (.), star (*) and double quote (") with an underscore (_)
-
-        :param tags: key/value pairs of tags
-        :return: None
-        """
-        for key in tags.keys():
-            self.labels[LABEL_RE.sub("_", compat.text_type(key))] = encoding.keyword_field(compat.text_type(tags[key]))
-
     def set_success(self):
         self.outcome = "success"
 
     def set_failure(self):
         self.outcome = "failure"
 
+    @staticmethod
+    def get_dist_tracing_id():
+        return "%016x" % random.getrandbits(64)
+
 
 class Transaction(BaseSpan):
-    def __init__(self, tracer, transaction_type="custom", trace_parent=None, is_sampled=True, start=None):
-        self.id = "%016x" % random.getrandbits(64)
+    def __init__(
+        self, tracer, transaction_type="custom", trace_parent=None, is_sampled=True, start=None, sample_rate=None
+    ):
+        self.id = self.get_dist_tracing_id()
         self.trace_parent = trace_parent
         if start:
             self.timestamp = self.start_time = start
@@ -157,7 +144,8 @@ class Transaction(BaseSpan):
         self.dropped_spans = 0
         self.context = {}
 
-        self.is_sampled = is_sampled
+        self._is_sampled = is_sampled
+        self.sample_rate = sample_rate
         self._span_counter = 0
         self._span_timers = defaultdict(Timer)
         self._span_timers_lock = threading.Lock()
@@ -181,8 +169,9 @@ class Transaction(BaseSpan):
             self._transaction_metrics.timer(
                 "transaction.duration",
                 reset_on_collect=True,
+                unit="us",
                 **{"transaction.name": self.name, "transaction.type": self.transaction_type}
-            ).update(self.duration)
+            ).update(int(self.duration * 1000000))
         if self._breakdown:
             for (span_type, span_subtype), timer in compat.iteritems(self._span_timers):
                 labels = {
@@ -192,15 +181,19 @@ class Transaction(BaseSpan):
                 }
                 if span_subtype:
                     labels["span.subtype"] = span_subtype
-                self._breakdown.timer("span.self_time", reset_on_collect=True, **labels).update(*timer.val)
+                val = timer.val
+                self._breakdown.timer("span.self_time", reset_on_collect=True, unit="us", **labels).update(
+                    int(val[0] * 1000000), val[1]
+                )
             labels = {"transaction.name": self.name, "transaction.type": self.transaction_type}
             if self.is_sampled:
                 self._breakdown.counter("transaction.breakdown.count", reset_on_collect=True, **labels).inc()
                 self._breakdown.timer(
                     "span.self_time",
                     reset_on_collect=True,
+                    unit="us",
                     **{"span.type": "app", "transaction.name": self.name, "transaction.type": self.transaction_type}
-                ).update(self.duration - self._child_durations.duration)
+                ).update(int((self.duration - self._child_durations.duration) * 1000000))
 
     def _begin_span(
         self,
@@ -325,6 +318,8 @@ class Transaction(BaseSpan):
             "sampled": self.is_sampled,
             "span_count": {"started": self._span_counter - self.dropped_spans, "dropped": self.dropped_spans},
         }
+        if self.sample_rate is not None:
+            result["sample_rate"] = float(self.sample_rate)
         if self.trace_parent:
             result["trace_id"] = self.trace_parent.trace_id
             # only set parent_id if this transaction isn't the root
@@ -340,6 +335,23 @@ class Transaction(BaseSpan):
         with self._span_timers_lock:
             self._span_timers[(span_type, span_subtype)].update(self_duration)
 
+    @property
+    def is_sampled(self):
+        return self._is_sampled
+
+    @is_sampled.setter
+    def is_sampled(self, is_sampled):
+        """
+        This should never be called in normal operation, but often is used
+        for testing. We just want to make sure our sample_rate comes out correctly
+        in tracestate if we set is_sampled to False.
+        """
+        self._is_sampled = is_sampled
+        if not is_sampled:
+            if self.sample_rate:
+                self.sample_rate = "0"
+                self.trace_parent.add_tracestate(constants.TRACESTATE.SAMPLE_RATE, self.sample_rate)
+
 
 class Span(BaseSpan):
     __slots__ = (
@@ -394,7 +406,7 @@ class Span(BaseSpan):
         :param start: timestamp, mostly useful for testing
         """
         self.start_time = start or _time_func()
-        self.id = "%016x" % random.getrandbits(64)
+        self.id = self.get_dist_tracing_id()
         self.transaction = transaction
         self.name = name
         self.context = context if context is not None else {}
@@ -439,6 +451,8 @@ class Span(BaseSpan):
             "duration": self.duration * 1000,  # milliseconds
             "outcome": self.outcome,
         }
+        if self.transaction.sample_rate is not None:
+            result["sample_rate"] = float(self.transaction.sample_rate)
         if self.sync is not None:
             result["sync"] = self.sync
         if self.labels:
@@ -462,7 +476,7 @@ class Span(BaseSpan):
         tracer = self.transaction.tracer
         timestamp = _time_func()
         self.duration = duration if duration is not None else (timestamp - self.start_time)
-        if not tracer.span_frames_min_duration or self.duration >= tracer.span_frames_min_duration:
+        if not tracer.span_frames_min_duration or self.duration >= tracer.span_frames_min_duration and self.frames:
             self.frames = tracer.frames_processing_func(self.frames)[skip_frames:]
         else:
             self.frames = None
@@ -564,11 +578,24 @@ class Tracer(object):
         """
         if trace_parent:
             is_sampled = bool(trace_parent.trace_options.recorded)
+            sample_rate = trace_parent.tracestate_dict.get(constants.TRACESTATE.SAMPLE_RATE)
         else:
             is_sampled = (
                 self.config.transaction_sample_rate == 1.0 or self.config.transaction_sample_rate > random.random()
             )
-        transaction = Transaction(self, transaction_type, trace_parent=trace_parent, is_sampled=is_sampled, start=start)
+            if not is_sampled:
+                sample_rate = "0"
+            else:
+                sample_rate = str(self.config.transaction_sample_rate)
+
+        transaction = Transaction(
+            self,
+            transaction_type,
+            trace_parent=trace_parent,
+            is_sampled=is_sampled,
+            start=start,
+            sample_rate=sample_rate,
+        )
         if trace_parent is None:
             transaction.trace_parent = TraceParent(
                 constants.TRACE_CONTEXT_VERSION,
@@ -576,15 +603,10 @@ class Tracer(object):
                 transaction.id,
                 TracingOptions(recorded=is_sampled),
             )
+            transaction.trace_parent.add_tracestate(constants.TRACESTATE.SAMPLE_RATE, sample_rate)
         execution_context.set_transaction(transaction)
         return transaction
 
-    def _should_ignore(self, transaction_name):
-        for pattern in self._ignore_patterns:
-            if pattern.search(transaction_name):
-                return True
-        return False
-
     def end_transaction(self, result=None, transaction_name=None, duration=None):
         """
         End the current transaction and queue it for sending
@@ -605,6 +627,12 @@ class Tracer(object):
             self.queue_func(TRANSACTION, transaction.to_dict())
         return transaction
 
+    def _should_ignore(self, transaction_name):
+        for pattern in self._ignore_patterns:
+            if pattern.search(transaction_name):
+                return True
+        return False
+
 
 class capture_span(object):
     __slots__ = (
@@ -628,7 +656,6 @@ class capture_span(object):
         extra=None,
         skip_frames=0,
         leaf=False,
-        tags=None,
         labels=None,
         span_subtype=None,
         span_action=None,
@@ -643,14 +670,6 @@ class capture_span(object):
         self.extra = extra
         self.skip_frames = skip_frames
         self.leaf = leaf
-        if tags and not labels:
-            warnings.warn(
-                'The tags argument to capture_span is deprecated, use "labels" instead',
-                category=DeprecationWarning,
-                stacklevel=2,
-            )
-            labels = tags
-
         self.labels = labels
         self.start = start
         self.duration = duration
@@ -695,7 +714,7 @@ class capture_span(object):
                         # could happen if the exception has __slots__
                         pass
             except LookupError:
-                logger.info("ended non-existing span %s of type %s", self.name, self.type)
+                logger.debug("ended non-existing span %s of type %s", self.name, self.type)
 
 
 def label(**labels):
@@ -712,18 +731,6 @@ def label(**labels):
         transaction.label(**labels)
 
 
-@deprecated("elasticapm.label")
-def tag(**tags):
-    """
-    Tags current transaction. Both key and value of the label should be strings.
-    """
-    transaction = execution_context.get_transaction()
-    if not transaction:
-        error_logger.warning("Ignored tags %s. No transaction currently active.", ", ".join(tags.keys()))
-    else:
-        transaction.tag(**tags)
-
-
 def set_transaction_name(name, override=True):
     """
     Sets the name of the transaction
@@ -799,6 +806,16 @@ def get_transaction_id():
     return transaction.id
 
 
+def get_trace_parent_header():
+    """
+    Return the trace parent header for the current transaction.
+    """
+    transaction = execution_context.get_transaction()
+    if not transaction or not transaction.trace_parent:
+        return
+    return transaction.trace_parent.to_string()
+
+
 def get_trace_id():
     """
     Returns the current trace ID
